library jaguar_mux.example.simple.server;

import 'package:jaguar/jaguar.dart';
import 'package:jaguar_mux/jaguar_mux.dart';
import 'package:jaguar/interceptors.dart';

class WrapSimpleInterceptor implements RouteWrapper<SimpleInterceptor> {
  final String name;

  /// Id of the interceptor
  String get id => '';

  Map<Symbol, MakeParam> get makeParams => {};

  const WrapSimpleInterceptor(this.name);

  SimpleInterceptor createInterceptor() => new SimpleInterceptor(name);
}

class SimpleInterceptor extends Interceptor {
  final String name;

  SimpleInterceptor(this.name);

  String pre() => 'input';

  void post() => print('post $name');
}

Map versionRoute() => {'v': 0.1};

String getUserById(String id) => id;

Map getQueryParams(
    @Input(SimpleInterceptor) String input, @InputHeader('hello') String hello,
    {String name, String password}) {
  return {
    'n': name,
    'p': password,
    'h': hello,
  };
}

main() async {
  final muxer = new MuxBuilder();

  muxer
      .get('/encode/maptojson', () => {'name': 'jaguar'})
      .wrap(new WrapEncodeMapToJson());

  muxer
      .get('/wrap/simpleinterceptor', () => {'name': 'jaguar'})
      .wrap(new WrapSimpleInterceptor('hello'));

  muxer.get('/user/:id', getUserById);

  muxer.get('/version', versionRoute);

  muxer
      .get('/query/params', getQueryParams)
      .wrap(new WrapSimpleInterceptor('hello'));

  Mux mux = muxer.build();

  Configuration configuration = new Configuration();
  configuration.addApi(mux);

  await serve(configuration);
}
